/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Gmail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Doron Rosenberg.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
   XXX:
     update checker in prefs
*/

const nsIGMNotifierProgressListener = Components.interfaces.nsIGMNotifierProgressListener;

function gm_notifier(){
  this.observer;
  this.gm_browser;
  this.user_name;
  this.password;
  this.is_logged_in = false;
  this.has_ever_logged_in = false;
  this.login_window;
  this.reset_counter = false;
 
  this.gm_prefs = new gm_prefs();
  
  this.labels = new Array();
  
  this.nsIGmailNotifier = null;
  this.nsIGMNotifierListener = null;

  // init prefs
  var arePrefsSet = this.gm_prefs.getBoolPref(this.gm_prefs.PREF_LOAD_RESET_COUNTER);
  if (arePrefsSet == null){
    //prefs haven't been set yet
    this.initPref(this.gm_prefs.PREF_STATUSBAR_ENABLED , "bool", true);
    this.initPref(this.gm_prefs.PREF_UPDATE_INTERVAL, "int", 10);
    this.initPref(this.gm_prefs.PREF_DEFAULT_USER, "char", "");
    this.initPref(this.gm_prefs.PREF_REMEMBER_PASSWORD, "bool", false);
    this.initPref(this.gm_prefs.PREF_LOAD_LOCATION, "int", 0);
    this.initPref(this.gm_prefs.PREF_LOAD_RESET_COUNTER, "bool", false);
    this.initPref(this.gm_prefs.PREF_AUTOLOGIN_ENABLED, "bool", false);
    this.initPref(this.gm_prefs.PREF_IS_LOGGED_IN, "bool", false);
    this.initPref(this.gm_prefs.PREF_NOTIFICATIONS_ENABLED, "bool", true);
    this.initPref(this.gm_prefs.PREF_SOUNDNOTIFICATIONS_ENABLED, "bool", false);
  }

  this.reset_counter = this.gm_prefs.getBoolPref(this.gm_prefs.PREF_LOAD_RESET_COUNTER);

  // add pref observers
  this.PrefChangeObserver = {
    observe: function(aSubject, aTopic, aData)
    {
      gGMailNotifier.prefChanged(aData);
    }
  };

  this.gm_prefs.addObserver("gm-notifier", this.PrefChangeObserver);
}

gm_notifier.prototype.initPref = function (aPrefName, aPrefType, aDefaultValue){
  switch (aPrefType) {
    case "bool" :
      var prefExists = this.gm_prefs.getBoolPref(aPrefName);
      if (prefExists == null)
        this.gm_prefs.setBoolPref(aPrefName, aDefaultValue);
      break;

    case "int" :
      var prefExists = this.gm_prefs.getIntPref(aPrefName);
      if (prefExists == null)
        this.gm_prefs.setIntPref(aPrefName, aDefaultValue);
      break;

    case "char" :
      var prefExists = this.gm_prefs.getCharPref(aPrefName);
      if (prefExists == null)
        this.gm_prefs.setCharPref(aPrefName, aDefaultValue);
      break;
  }
}

gm_notifier.prototype.prefChanged = function (aPrefName){
  switch(aPrefName){
    case gGMailNotifier.gm_prefs.PREF_STATUSBAR_ENABLED:
      gGMailNotifier.showStatusbarItem();
      break;

    case gGMailNotifier.gm_prefs.PREF_LOAD_RESET_COUNTER:
      gGMailNotifier.reset_counter = 
        gGMailNotifier.gm_prefs.getBoolPref(gGMailNotifier.gm_prefs.PREF_LOAD_RESET_COUNTER);
      break;
  }
}

gm_notifier.prototype.onload = function (){
  // this is false if we are in ff's customize window
  if(document.getElementById("gm-notifier-statusbar")) {

    // add the listener on startup and init the xpcom clas
    try {
      gGMailNotifier.nsIGmailNotifier = Components.classes["@mozilla.org/GMailNotifier;1"]
                                                  .getService(Components.interfaces.nsIGMNotifierService);
      gGMailNotifier.nsIGMNotifierListener = new gGMailNotifier.nsIGMNotifierProgressListener()
      gGMailNotifier.nsIGmailNotifier.addListener(gGMailNotifier.nsIGMNotifierListener);

      document.getElementById("gm-notifier-statusbar").collapsed = !gGMailNotifier.gm_prefs.getBoolPref(gGMailNotifier.gm_prefs.PREF_STATUSBAR_ENABLED);
      document.getElementById("gm-context-menu-statusbar-item").setAttribute("checked", gGMailNotifier.gm_prefs.getBoolPref(gGMailNotifier.gm_prefs.PREF_STATUSBAR_ENABLED));
    } catch (e) {
    }

    window.removeEventListener("load", gGMailNotifier.onload, false);
  }
}

gm_notifier.prototype.setLoginStatus = function (aStatusNum){
  if (this.login_window && this.login_window.document)
    this.login_window.setStatus(aStatusNum);
}

gm_notifier.prototype.onFinishLoginLoad = function (){
  // set login flags
  this.trying_to_login = false;
  this.has_ever_logged_in = true;

  // update the login window
  this.setLoginStatus(2);
  // update the pref
  this.gm_prefs.setBoolPref(this.gm_prefs.PREF_IS_LOGGED_IN, true);

  // set UI labels         
  this.updateLabels(false);

  document.getElementById("gm-context-menu-logout").setAttribute("disabled", false);
}

gm_notifier.prototype.updateLabels = function (aIconClicked){
  // update the UI objects
  document.getElementById("gm-notifier-statusbar").setAttribute("logged-in", this.is_logged_in);

  if (document.getElementById("gm-notifier-toolbar-item"))
    document.getElementById("gm-notifier-toolbar-item").setAttribute("logged-in", this.is_logged_in);

  if (this.is_logged_in){

    if (aIconClicked && this.reset_counter)
      this.nsIGmailNotifier.setResetState(true);

    var unread = this.nsIGmailNotifier.getUnreadCount();

  
    if (document.getElementById("gm-notifier-toolbar-item"))
      document.getElementById("gm-notifier-toolbar-item").setAttribute("unread", unread);

    if (document.getElementById("gm-notifier-statusbar")) {
      document.getElementById("gm-notifier-statusbar").setAttribute("label", unread);
    }
  } else {
    if (document.getElementById("gm-notifier-toolbar-item"))
      document.getElementById("gm-notifier-toolbar-item").setAttribute("unread", "");

    if (document.getElementById("gm-notifier-statusbar"))
      document.getElementById("gm-notifier-statusbar").setAttribute("label", "");
  }
}

gm_notifier.prototype.init = function (aUserName, aPassword){
  this.user_name = aUserName;
  this.password = aPassword;

  this.setLoginStatus(1);
  this.initiateLogin();
}

gm_notifier.prototype.login = function (aEvent){
  // on windows, right click calls this method
  if (aEvent && (aEvent.button == 2))
    return;

  if (!this.is_logged_in) {
    this.openLoginWindow();
  } else {
    // middle click should open in new tab - from statusbar
    if ((this.is_logged_in) && (aEvent) && (aEvent.button == 1))
      this.loadGM(1);
    else
      this.loadGM();
  }  
}

gm_notifier.prototype.openLoginWindow = function (){
  this.login_window = window.openDialog("chrome://gm-notifier/content/gm-login.xul", "_blank", "chrome,resizable=no,dependent=yes");
}

gm_notifier.prototype.nsIGMNotifierProgressListener = function(){
  return ({
  
    onStateChange : function (aState){

      if (document.getElementById("gm-notifier-toolbar-item"))
        document.getElementById("gm-notifier-toolbar-item").removeAttribute("loading");
    
      if (aState == nsIGMNotifierProgressListener.LOGIN_INITIATED){
        if (document.getElementById("gm-notifier-toolbar-item"))
          document.getElementById("gm-notifier-toolbar-item").setAttribute("loading", "true");
      }

      if (aState == nsIGMNotifierProgressListener.NO_NEW_MAIL){
      }

      if (aState == nsIGMNotifierProgressListener.NEW_MAIL) {
      }

      if (aState == nsIGMNotifierProgressListener.LOGIN_FAILED) {
        gGMailNotifier.setLoginStatus(3);
        gGMailNotifier.is_logged_in = false;
        gGMailNotifier.updateLabels(false);
        document.getElementById("gm-context-menu-logout").setAttribute("disabled", true);
      }  

      if (aState == nsIGMNotifierProgressListener.LOGIN_SUCCESS) {
        gGMailNotifier.is_logged_in = true;
        gGMailNotifier.onFinishLoginLoad();
      } 

      if (aState == nsIGMNotifierProgressListener.LOGOUT) {
        if (gGMailNotifier.is_logged_in){
          gGMailNotifier.is_logged_in = false;
   
          gGMailNotifier.gm_prefs.setBoolPref(gGMailNotifier.gm_prefs.PREF_IS_LOGGED_IN, false);

          document.getElementById("gm-context-menu-logout").setAttribute("disabled", true);
          gGMailNotifier.updateLabels(false);
        }
      }

      // called when notification gets clicked
      if (aState == nsIGMNotifierProgressListener.LOAD_GMAIL) {
        gGMailNotifier.loadGM();
      } 
    },

    QueryInterface : function(aIID) {
      if (aIID.equals(nsIGMNotifierProgressListener) ||
          aIID.equals(Components.interfaces.nsISupports))
        return this;
      throw Components.results.NS_NOINTERFACE;
    }            
  }  
  );
}

gm_notifier.prototype.initiateLogin = function (){
  gGMailNotifier.nsIGmailNotifier.logItem("initiateLoging: window is " + window);
  gGMailNotifier.nsIGmailNotifier.initLogin(gGMailNotifier.user_name, gGMailNotifier.password, window);
  gGMailNotifier.has_ever_logged_in = true;
}

gm_notifier.prototype._talkToServer = function (){
  gGMailNotifier.nsIGmailNotifier.initLogin(gGMailNotifier.user_name, gGMailNotifier.password, window);
}

gm_notifier.prototype.callback_4 = function (aData, aRequest){

  var val = aData.match(/\["ds",.*\]/);

  // handle gmail labels
  var x = aData.indexOf("D([\"ct\"");
  var y = aData.indexOf("D([\"ts\"");
  var z = aData.substr(x+2, y-x-5);

  var myAr = eval(z)[1];

  // new array, so that removed labels get removed.
  /*
  var new_labels_array = new Array();

  for (var run = 0; run < myAr.length; run++){
    var l_name = myAr[run][0];
    var l_unread = myAr[run][1];

    if (gGMailNotifier.labels[l_name])
      new_labels_array[l_name] = new gm_label(l_name, l_unread, gGMailNotifier.labels[l_name].unread);
    else 
      new_labels_array[l_name] = new gm_label(l_name, l_unread);
  }
  
  gGMailNotifier.labels = new_labels_array;

  var myTooltipRows = document.getElementById("gm-tooltip-row");
  */
  // clear current entries
/*  for (var run = myTooltipRows.childNodes.length; run--; run > 0){
    myTooltipRows.removeChild(myTooltipRows.childNodes.item(run))
  }

  for (var labelName in gGMailNotifier.labels) {
    var myRow = document.createElement("row");
    
    var myLabel = document.createElement("label");
    myLabel.setAttribute("value", labelName);
    myRow.appendChild(myLabel);

    myLabel = document.createElement("label");
    
    if (gGMailNotifier.labels[labelName].prev_unread && (gGMailNotifier.labels[labelName].unread > gGMailNotifier.labels[labelName].prev_unread) )
      myLabel.setAttribute("value", gGMailNotifier.labels[labelName].unread + " (" + (gGMailNotifier.labels[labelName].unread - gGMailNotifier.labels[labelName].prev_unread) + ")");
    else
      myLabel.setAttribute("value", gGMailNotifier.labels[labelName].unread);
    myRow.appendChild(myLabel);
    
    myTooltipRows.appendChild(myRow);
  }*/

  gGMailNotifier.onFinishLoginLoad();     
}

gm_notifier.prototype.fillInTooltip = function(aTooltipElement){
  document.getElementById("gm-notifier-tooltip-logged-in").collapsed = !this.is_logged_in;
  document.getElementById("gm-notifier-tooltip-logged-out").collapsed = this.is_logged_in;

  if (this.is_logged_in) {
    document.getElementById("gm-notifier-tooltip-username").value = this.nsIGmailNotifier.getUserName();
    
    document.getElementById("gm-notifier-tooltip-unread").value = 
      this.getFormattedString("TooltipUnread", [this.nsIGmailNotifier.getUnreadCount()]);

    document.getElementById("gm-notifier-tooltip-quota").value =
      this.getFormattedString("TooltipQuota", [this.nsIGmailNotifier.getUsedMB(), this.nsIGmailNotifier.getSpaceUsed()]); 
  } else {
  }
}

gm_notifier.prototype.storeLoginDetails = function(aStoreInfo){
  var url = "chrome://gm-notifier/";

  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance();

  if (passwordManager) {
    passwordManager = passwordManager.QueryInterface(Components.interfaces.nsIPasswordManager);

    try{
      passwordManager.removeUser(url, this.user_name);
    } catch (e) {  
    }
  
    if (aStoreInfo) {
      this.gm_prefs.setBoolPref("gm-notifier.users.remember-password", true);
      this.gm_prefs.setCharPref("gm-notifier.users.default", this.user_name);
      passwordManager.addUser(url, this.user_name, this.password);
    }  
  }    
}

gm_notifier.prototype.getLoginDetails = function(aUsername){
  var url = "chrome://gm-notifier/";

  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance(Components.interfaces.nsIPasswordManagerInternal);
  var host = {value:""};
  var user =  {value:""};
  var password = {value:""}; 

  try {
    passwordManager.findPasswordEntry(url, aUsername, "", host, user, password);
  } catch(e){
  }  

  return password.value;
}

gm_notifier.prototype.showStatusbarItem = function (aShowItem){
   if (aShowItem != null){
     this.gm_prefs.setBoolPref(this.gm_prefs.PREF_STATUSBAR_ENABLED, aShowItem);
   }

   var prefValue = aShowItem ? aShowItem : this.gm_prefs.getBoolPref(this.gm_prefs.PREF_STATUSBAR_ENABLED);

   document.getElementById("gm-notifier-statusbar").collapsed = !prefValue;
   document.getElementById("gm-context-menu-statusbar-item").setAttribute("checked",  prefValue);
}

gm_notifier.prototype.loadPrefWindow = function (){
  window.openDialog("chrome://gm-notifier/content/gm-preferences.xul", "_blank", "chrome,resizable=no,dependent=yes")
}

// loadGM location is determined by gm-notifier.loadgm.location
// 0 - current tab
// 1 - new tab
// 2 - new window
// 3 - new unfocused tab
gm_notifier.prototype.loadGM = function (aOverride){
   gm_dumpString("\n Load Gmail called");

  const gm_url = "https://gmail.google.com/gmail";
  var location = aOverride ? aOverride : this.gm_prefs.getIntPref(this.gm_prefs.PREF_LOAD_LOCATION);

  if (location == 2){
    window.open(gm_url);
  } else if ((location == 1) || (location == 3)){
    var myTab = getBrowser().addTab(gm_url, null, null);

    // focus tab only if location is 1
    if (location == 1)
      getBrowser().selectedTab = myTab;
  } else {
    getBrowser().loadURI(gm_url);
  }

  // reset labels
  this.updateLabels(true);
}

gm_notifier.prototype.checkNow = function (){
  if (this.is_logged_in){
    //this._talkToServer();
    this.nsIGmailNotifier.checkNow();
  } else {
    this.openLoginWindow();
  }
}

gm_notifier.prototype.logout = function (){
  this.nsIGmailNotifier.logout();
}

gm_notifier.prototype.openMiddleClick = function (aEvent){
  // middle click should open in new tab
  if ((this.is_logged_in) && (aEvent) && (aEvent.button == 1))
    this.loadGM(1);
}

gm_notifier.prototype.getString = function (aName){
  var strbundle = document.getElementById("gm-notifier-stringbundle");
  return strbundle.getString(aName);
}

gm_notifier.prototype.getFormattedString = function (aName, aStrArray){
  var strbundle = document.getElementById("gm-notifier-stringbundle");
  return strbundle.getFormattedString(aName, aStrArray);
}

// shutdown
gm_notifier.prototype.onunload = function (){
  if(document.getElementById("gm-notifier-statusbar")) {
    /* cleanup time! */

    // remove the listener
    if (gGMailNotifier.nsIGmailNotifier)
      gGMailNotifier.nsIGmailNotifier.removeListener(gGMailNotifier.nsIGMNotifierListener);

    // remove the pref observer
    if (gGMailNotifier.gm_prefs)
      gGMailNotifier.gm_prefs.removeObserver("gm-notifier", gGMailNotifier.PrefChangeObserver);
  }

  // last, but not least, clear the whole object
  gGMailNotifier = null;
}

var gGMailNotifier = new gm_notifier();

// gm labels
function gm_label(aName, aUnread, aPrevUnread) {
  this.name = aName;
  this.unread = aUnread;
  
  this.prev_unread = null;
  
  if (aPrevUnread)
    this.prev_unread = aPrevUnread;
}

function gm_dumpString(aMsg){
  if (true)
    dump(aMsg);
}

window.addEventListener("load", gGMailNotifier.onload, false);
window.addEventListener("unload", gGMailNotifier.onunload, false);